/* 
 * Copyright (C) 2015 Martin Gräßlin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DPMS_CLIENT_PROTOCOL_H
#define DPMS_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_dpms_manager;
struct org_kde_kwin_dpms;

extern const struct wl_interface org_kde_kwin_dpms_manager_interface;
extern const struct wl_interface org_kde_kwin_dpms_interface;

#define ORG_KDE_KWIN_DPMS_MANAGER_GET	0

static inline void
org_kde_kwin_dpms_manager_set_user_data(struct org_kde_kwin_dpms_manager *org_kde_kwin_dpms_manager, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_kwin_dpms_manager, user_data);
}

static inline void *
org_kde_kwin_dpms_manager_get_user_data(struct org_kde_kwin_dpms_manager *org_kde_kwin_dpms_manager)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_kwin_dpms_manager);
}

static inline void
org_kde_kwin_dpms_manager_destroy(struct org_kde_kwin_dpms_manager *org_kde_kwin_dpms_manager)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_dpms_manager);
}

static inline struct org_kde_kwin_dpms *
org_kde_kwin_dpms_manager_get(struct org_kde_kwin_dpms_manager *org_kde_kwin_dpms_manager, struct wl_output *output)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) org_kde_kwin_dpms_manager,
			 ORG_KDE_KWIN_DPMS_MANAGER_GET, &org_kde_kwin_dpms_interface, NULL, output);

	return (struct org_kde_kwin_dpms *) id;
}

#ifndef ORG_KDE_KWIN_DPMS_MODE_ENUM
#define ORG_KDE_KWIN_DPMS_MODE_ENUM
enum org_kde_kwin_dpms_mode {
	ORG_KDE_KWIN_DPMS_MODE_ON = 0,
	ORG_KDE_KWIN_DPMS_MODE_STANDBY = 1,
	ORG_KDE_KWIN_DPMS_MODE_SUSPEND = 2,
	ORG_KDE_KWIN_DPMS_MODE_OFF = 3,
};
#endif /* ORG_KDE_KWIN_DPMS_MODE_ENUM */

/**
 * org_kde_kwin_dpms - Dpms for a wl_output
 * @supported: Event indicating whether DPMS is supported on the
 *	wl_output
 * @mode: Event indicating used DPMS mode
 * @done: All changes are pushed
 *
 * This interface provides information about the VESA DPMS state for a
 * wl_output. It gets created through the request get on the
 * org_kde_kwin_dpms_manager interface.
 *
 * On creating the resource the server will push whether DPSM is supported
 * for the output, the currently used DPMS state and notifies the client
 * through the done event once all states are pushed. Whenever a state
 * changes the set of changes is committed with the done event.
 */
struct org_kde_kwin_dpms_listener {
	/**
	 * supported - Event indicating whether DPMS is supported on the
	 *	wl_output
	 * @supported: Boolean value whether DPMS is supported (1) for
	 *	the wl_output or not (0)
	 *
	 * This event gets pushed on binding the resource and indicates
	 * whether the wl_output supports DPMS. There are operation modes
	 * of a Wayland server where DPMS might not make sense (e.g. nested
	 * compositors).
	 */
	void (*supported)(void *data,
			  struct org_kde_kwin_dpms *org_kde_kwin_dpms,
			  uint32_t supported);
	/**
	 * mode - Event indicating used DPMS mode
	 * @mode: The new currently used mode
	 *
	 * This mode gets pushed on binding the resource and provides the
	 * currently used DPMS mode. It also gets pushed if DPMS is not
	 * supported for the wl_output, in that case the value will be On.
	 *
	 * The event is also pushed whenever the state changes.
	 */
	void (*mode)(void *data,
		     struct org_kde_kwin_dpms *org_kde_kwin_dpms,
		     uint32_t mode);
	/**
	 * done - All changes are pushed
	 *
	 * This event gets pushed on binding the resource once all other
	 * states are pushed.
	 *
	 * In addition it gets pushed whenever a state changes to tell the
	 * client that all state changes have been pushed.
	 */
	void (*done)(void *data,
		     struct org_kde_kwin_dpms *org_kde_kwin_dpms);
};

static inline int
org_kde_kwin_dpms_add_listener(struct org_kde_kwin_dpms *org_kde_kwin_dpms,
			       const struct org_kde_kwin_dpms_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) org_kde_kwin_dpms,
				     (void (**)(void)) listener, data);
}

#define ORG_KDE_KWIN_DPMS_SET	0
#define ORG_KDE_KWIN_DPMS_RELEASE	1

static inline void
org_kde_kwin_dpms_set_user_data(struct org_kde_kwin_dpms *org_kde_kwin_dpms, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_kwin_dpms, user_data);
}

static inline void *
org_kde_kwin_dpms_get_user_data(struct org_kde_kwin_dpms *org_kde_kwin_dpms)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_kwin_dpms);
}

static inline void
org_kde_kwin_dpms_destroy(struct org_kde_kwin_dpms *org_kde_kwin_dpms)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_dpms);
}

static inline void
org_kde_kwin_dpms_set(struct org_kde_kwin_dpms *org_kde_kwin_dpms, uint32_t mode)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_dpms,
			 ORG_KDE_KWIN_DPMS_SET, mode);
}

static inline void
org_kde_kwin_dpms_release(struct org_kde_kwin_dpms *org_kde_kwin_dpms)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_kwin_dpms,
			 ORG_KDE_KWIN_DPMS_RELEASE);

	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_dpms);
}

#ifdef  __cplusplus
}
#endif

#endif
