/* 
 * Copyright © 2015 Sebastian Kügler <sebas@kde.org>
 * Copyright © 2008-2011 Kristian Høgsberg
 * Copyright © 2010-2011 Intel Corporation
 * Copyright © 2012-2013 Collabora, Ltd.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#ifndef OUTPUTMANAGEMENT_SERVER_PROTOCOL_H
#define OUTPUTMANAGEMENT_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_outputmanagement;
struct org_kde_kwin_outputconfiguration;

extern const struct wl_interface org_kde_kwin_outputmanagement_interface;
extern const struct wl_interface org_kde_kwin_outputconfiguration_interface;

/**
 * org_kde_kwin_outputmanagement - list connected, but disabled outputs
 * @create_configuration: change the shell surface position
 *
 * This interface allows to connected output devices for screen
 * configuration purposes.
 *
 * OutputManagement (wl_global) -------------------------- request: *
 * createConfiguration -> OutputConfiguration (wl_resource)
 *
 * OutputConfiguration (wl_resource) -------------------------- requests: *
 * enable(output, bool) * mode(output, mode_id) * transformation(output,
 * flag) * position(output, x, y) * apply
 *
 * events: * applied * failed
 *
 * The idea is that we register one OutputManagement as a global. When
 * KScreen wants to configure it calls createConfiguration which provides a
 * resource referencing an OutputConfiguration for one-time configuration.
 * That way we can easily group all requests and know on the server which
 * one belong together. On the OutputConfiguration it calls for each Output
 * whether it's enabled, which mode to use (by referencing the mode from
 * the list of announced modes) and the global position. Once all outputs
 * are configured that way, it calls apply. At that point and not earlier
 * the server emits a signal to KWin with the complete wanted configuration
 * which KWin can then try to apply. If that succeeds it emits the applied
 * signal, otherwise the failed. So even KScreen can know about it.
 *
 * That interface is nicely atomic (as KWin only gets the apply for
 * everything setup), it solves the 1:n mapping problem: each
 * OutputConfiguration is only a 1:1 mapping, and we do not need to copy
 * all the requests again by just referencing a mode id.
 */
struct org_kde_kwin_outputmanagement_interface {
	/**
	 * create_configuration - change the shell surface position
	 * @id: (none)
	 *
	 * Request an output configuration, which can be changed and then
	 * applied.
	 */
	void (*create_configuration)(struct wl_client *client,
				     struct wl_resource *resource,
				     uint32_t id);
};


/**
 * org_kde_kwin_outputconfiguration - list connected, but disabled
 *	outputs
 * @enable: enable or disable an output
 * @mode: Switch outputdevice to mode
 * @transform: Transform outputdevice
 * @position: Position output relative to global
 * @scale: Set the scaling factor for this outputdevice
 * @apply: apply the changes to the outputdevices
 *
 * outputconfiguration is a client-specific resource that can be used to
 * ask the compositor to apply changes to the outputdevices, the monitors
 * connected to the system. The client receives a list of outputdevices
 * from the registry. When it wants to apply new settings, it creates a
 * configuration object from the outputmanagement global, writes changes
 * through this object's enable, scale, transform and mode calls. It then
 * asks the compositor to apply these settings in an atomic fashion, for
 * example through Linux's DRM interface. The compositor signals back
 * whether the new settings have applied successfully or failed to apply.
 * outputdevices are updated after the hardware changes have been applied
 * and before the server side fires applied.
 */
struct org_kde_kwin_outputconfiguration_interface {
	/**
	 * enable - enable or disable an output
	 * @outputdevice: (none)
	 * @enable: request to enable or disable this output
	 *
	 * Mark the output as enabled or disabled.
	 */
	void (*enable)(struct wl_client *client,
		       struct wl_resource *resource,
		       struct wl_resource *outputdevice,
		       int32_t enable);
	/**
	 * mode - Switch outputdevice to mode
	 * @outputdevice: outputdevice this mode change applies to
	 * @mode_id: aspired mode's id
	 *
	 * Sets the mode for a given output by its mode size (width and
	 * height) and refresh rate.
	 */
	void (*mode)(struct wl_client *client,
		     struct wl_resource *resource,
		     struct wl_resource *outputdevice,
		     int32_t mode_id);
	/**
	 * transform - Transform outputdevice
	 * @outputdevice: outputdevice this transformation change applies
	 *	to
	 * @transform: transform enum
	 *
	 * Sets the transformation for a given output.
	 */
	void (*transform)(struct wl_client *client,
			  struct wl_resource *resource,
			  struct wl_resource *outputdevice,
			  int32_t transform);
	/**
	 * position - Position output relative to global
	 * @outputdevice: outputdevice this position applies to
	 * @x: position on the x-axis
	 * @y: position on the y-axis
	 *
	 * Sets the position for this output device in the global space.
	 * position describes the top-left border of the output in the
	 * global space. The left-most output's top-left border is 0, 0.
	 * There may be no gaps between outputs, the outputs have to be
	 * positioned stacked horizontally, vertically, or both.
	 */
	void (*position)(struct wl_client *client,
			 struct wl_resource *resource,
			 struct wl_resource *outputdevice,
			 int32_t x,
			 int32_t y);
	/**
	 * scale - Set the scaling factor for this outputdevice
	 * @outputdevice: outputdevice this mode change applies to
	 * @scale: scaling factor
	 *
	 * Sets the scaling factor for this outputdevice.
	 */
	void (*scale)(struct wl_client *client,
		      struct wl_resource *resource,
		      struct wl_resource *outputdevice,
		      int32_t scale);
	/**
	 * apply - apply the changes to the outputdevices
	 *
	 * Asks the compositor to apply changes to the outputdevices on
	 * the server side.
	 */
	void (*apply)(struct wl_client *client,
		      struct wl_resource *resource);
};

#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_APPLIED	0
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_FAILED	1

#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_APPLIED_SINCE_VERSION	1
#define ORG_KDE_KWIN_OUTPUTCONFIGURATION_FAILED_SINCE_VERSION	1

static inline void
org_kde_kwin_outputconfiguration_send_applied(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_OUTPUTCONFIGURATION_APPLIED);
}

static inline void
org_kde_kwin_outputconfiguration_send_failed(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_OUTPUTCONFIGURATION_FAILED);
}

#ifdef  __cplusplus
}
#endif

#endif
